function [psi,psi_four] = timestep_benchmark(psi,psi_four,t_start,pars,vecs)
% Perform one timestep of the double VoC method for the limit system

% Storage scheme:
% Psi (and all other vectors corresponding to two component functions) is a
% vector of size 8*M:
% Psi(    1:2*M) --> Psi^+_1 at the nodes x_(-M),...,x_(M-1)
% Psi(2*M+1:4*M) --> Psi^+_2 at the nodes x_(-M),...,x_(M-1)
% Psi(4*M+1:6*M) --> Psi^-_1 at the nodes x_(-M),...,x_(M-1)
% Psi(6*M+1:8*M) --> Psi^-_2 at the nodes x_(-M),...,x_(M-1)
% Thus, to be more precise, 
%   psi = (psi^+,psi^-)^T = (psi^+_1,psi^+_2,psi^-_1,psi^-_2)^T

M = size(psi,1)/8;

%% Apply potential and non-linearity
W_psi = potential_W(psi,pars);
g_0 = abs(psi(    1:2*M)).^2 + abs(psi(2*M+1:4*M)).^2 + ...
      abs(psi(4*M+1:6*M)).^2 + abs(psi(6*M+1:8*M)).^2;
zeta = repmat(g_0, 4,1) .* psi;

%% Compute theta and zeta_prime
% theta
temp = W_psi + pars.gamma_2 * zeta;
theta_plus_four = -1i*repmat(vecs.D,2,1) .* psi_four(1:4*M) ...
              -1i*plus_projector_C2(  fft_C2(temp(    1:4*M)), pars, vecs.proj );
theta_minus_four = +1i*repmat(vecs.D,2,1) .* psi_four(4*M+1:8*M) ...
              -1i*minus_projector_C2( fft_C2(temp(4*M+1:8*M)), pars, vecs.proj );
theta_plus  = ifft_C2( theta_plus_four );
theta_minus = ifft_C2( theta_minus_four );
% zeta prime
temp = conj(theta_plus(1:2*M) ) .* psi(    1:2*M) + conj(theta_plus(2*M+1:4*M) ) .* psi(2*M+1:4*M) + ...
       conj(theta_minus(1:2*M)) .* psi(4*M+1:6*M) + conj(theta_minus(2*M+1:4*M)) .* psi(6*M+1:8*M) ; 
temp = temp + conj(temp);
zeta_prime = repmat(g_0,4,1) .* [theta_plus;theta_minus] + repmat(temp,4,1) .* psi;

%% apply projectors + operators
% terms of order tau
temp = fft_2C2( W_psi + pars.gamma_2 * zeta );
I_1_plus_four  = -1i  * repmat( vecs.kappa_pl(:,3) ,2,1) ...
                     .*  plus_projector_C2( temp(    1:4*M), pars, vecs.proj );
I_1_minus_four = -1i  * repmat( vecs.kappa_mi(:,3) ,2,1) ...
                     .* minus_projector_C2( temp(4*M+1:8*M), pars, vecs.proj );
% terms of order tau^2 (W is time independent --> no dt W term)
temp = fft_2C2( potential_W([theta_plus;theta_minus],pars) + pars.gamma_2 * zeta_prime );
I_2_plus_four  = -1i  * repmat( vecs.int_d_0_pl(:,3) ,2,1) ...
                     .*  plus_projector_C2( temp(    1:4*M), pars, vecs.proj );
I_2_minus_four = -1i  * repmat( vecs.int_d_0_mi(:,3) ,2,1) ...
                     .* minus_projector_C2( temp(4*M+1:8*M), pars, vecs.proj );

%% Calculate psi_new = approximation after one more time step
% Calculate the linear part exp(mp i tau D) psi in Fourier space, add all
% integrals (they are in Fourier space) and perform an inverse Fourier
% transform in the end
psi_four = [repmat(vecs.expD_minus,2,1); ...
            repmat(vecs.expD_plus, 2,1)].* psi_four ...
            + [I_1_plus_four;I_1_minus_four] ...
            + [I_2_plus_four;I_2_minus_four];
   
psi = ifft_2C2(psi_four);
end

